/* File de processus */
/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "proc_struct.h"

#include <stddef.h>

/* Tous les appels a proc_queue doivent se faire interruptions desactivees. */

void proc_queue_init(struct proc_queue_struct *proc_queue)
{
  proc_queue->first = NULL;
  proc_queue->last = NULL;
}

void proc_queue_push(struct proc_queue_struct *proc_queue, struct proc_struct *proc)
{
  proc->queue_previous = proc_queue->last;
  proc->queue_next = NULL;
  if (proc_queue->last)
    proc_queue->last->queue_next = proc;
  else
    proc_queue->first = proc;
  proc_queue->last = proc;
}

struct proc_struct *proc_queue_pop(struct proc_queue_struct *proc_queue)
{
  struct proc_struct *proc;

  if (!(proc = proc_queue->first))
    return NULL;
  proc_queue->first = proc->queue_next;
  if (proc->queue_next)
    proc->queue_next->queue_previous = NULL;
  else
    proc_queue->last = NULL;
  return proc;
}

/* Renvoie la tete de la file et replace celle-ci en queue de file. */
struct proc_struct *proc_queue_rotate(struct proc_queue_struct *proc_queue)
{
  struct proc_struct *proc;

  if (!(proc = proc_queue->first))
    return NULL;
  if (!proc->queue_next)
    return proc;
  proc_queue->first = proc->queue_next;
  proc->queue_next->queue_previous = NULL;
  proc->queue_previous = proc_queue->last;
  proc->queue_next = NULL;
  proc_queue->last->queue_next = proc;
  proc_queue->last = proc;
  return proc;
}

void proc_queue_remove(struct proc_queue_struct *proc_queue, struct proc_struct *proc)
{
  if (proc->queue_previous)
    proc->queue_previous->queue_next = proc->queue_next;
  else
    proc_queue->first = proc->queue_next;
  if (proc->queue_next)
    proc->queue_next->queue_previous = proc->queue_previous;
  else
    proc_queue->last = proc->queue_previous;
}
