/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <multiboot.h>

#include "util.h"

void __attribute__ ((noreturn)) die(const char *msg);

static multiboot_header_t *lookup_multiboot_header(void *start)
{
  unsigned long *p;

  p = start;
  while ((void *)p < start + 8192) {
    if (*p == MULTIBOOT_MAGIC)
      return (multiboot_header_t *)p;
    p++;
  }
  return NULL;
}

void *prepare_image(void *start)
{
  multiboot_header_t *multiboot_header;
  unsigned long header_addr, load_addr, load_end_addr, bss_end_addr, entry_addr;

  if (!(multiboot_header = lookup_multiboot_header(start)))
    goto error;
  if (multiboot_header->magic + multiboot_header->flags + multiboot_header->checksum)
    goto error;
  if (!(multiboot_header->flags & 0x00010000))
    goto error;
  header_addr = multiboot_header->header_addr;
  load_addr = multiboot_header->load_addr;
  load_end_addr = multiboot_header->load_end_addr;
  bss_end_addr = multiboot_header->bss_end_addr;
  entry_addr = multiboot_header->entry_addr;
  if (header_addr != (unsigned long)multiboot_header)
    memmove((void *)load_addr, (void *)multiboot_header - (header_addr - load_addr), load_end_addr - load_addr);
  memset((void *)load_end_addr, 0, bss_end_addr - load_end_addr);
  return (void *)entry_addr;

  error:
    die("Invalid kernel image");
}
