/* Code de decompression du systeme compresse */
/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/* Au demarrage, eax contient la taille du code charge par le secteur de boot.
 * Une pile de 4096 octets est utilisee, ce qui semble suffisant pour les
 * routines de decompression. */

#define STARTUP_START 0x10000
#define KERNEL_START 0x100000

#define STACK_SIZE 4096

	.text

.globl _start
.type _start,@function
.align 4
_start:
	movl	%eax, %ebx	/* taille du code charge */

	/* deplacement du systeme */
	leal	edata, %eax
	leal	_start, %ecx
	subl	%ecx, %eax	/* taille du code de startup charge */
	subl	%eax, %ebx	/* taille du systeme compresse */
	movl	%ebx, %eax
	subl	$1, %eax
	leal	edata, %esi
	addl	%eax, %esi	/* source */
	leal	end, %edi
	addl	%eax, %edi	/* destination */
	movl	%ebx, %ecx	/* nombre d'octets a deplacer */
	std			/* sens decroissant */
	rep
	movsb

	/* initialisation du BSS */
	leal	edata, %edi
	leal	end, %ecx
	subl	%edi, %ecx
	xorl	%eax, %eax
	cld
	rep
	stosb

	/* initialisation de la pile */
	lss	stack_top, %esp	/* initialisation de la pile */
	pushl	%ebx		/* taille du systeme compresse */

	/* ecriture d'un message */
	leal	msg, %ebx
	call	write_str

	/* decompression du systeme */
	pushl	%ecx
	pushl	$KERNEL_START
	pushl	8(%esp)
	call	decompress_kernel
	addl	$8, %esp
	popl	%ecx
	call	new_line

	/* preparation de l'image */
	pushl	$KERNEL_START
	call	prepare_image
	addl	$4, %esp

	/* saut vers le noyau */
	movl	%eax, %edx
	xorl	%eax, %eax
	jmp	*%edx

/* Ecrit une chaine a la position du curseur.
 *   ebx: adresse de la chaine a ecrire
 * retour:
 *   ecx: position du curseur
 * modifie: eax, ebx, dx */
.align 4
write_str:
	call	get_cursor_pos	/* lecture de la position du curseur */
	movb	$0x07, %ah
	shll	$1, %ecx
	addl	$0xb8000, %ecx
	jmp	2f
1:	movw	%ax, (%ecx)
	incl	%ebx
	addl	$2, %ecx
2:	movb	(%ebx), %al
	test	%al, %al
	jnz	1b
	subl	$0xb8000, %ecx
	shrl	$1, %ecx
	call	set_cursor_pos	/* ecriture de la position du curseur */
	ret

/* Positionne le curseur au debut de la ligne suivante (sans defilement).
 *   ecx: position courante
 * retour:
 *   ecx: nouvelle position
 * modifie: ebx */
.align 4
new_line:
	movl	%ecx, %ebx
	xorl	%ecx, %ecx
1:	addl	$80, %ecx
	cmpl	%ebx, %ecx
	jbe	1b
	call	set_cursor_pos	/* ecriture de la position du curseur */
	ret

/* Lit la position du curseur.
 * retour:
 *   ecx: position du curseur
 * modifie: dx */
.align 4
get_cursor_pos:
	pushw	%ax
	xorl	%ecx, %ecx
	movb	$0x0e, %al
	movw	$0x03d4, %dx
	outb	%al, %dx
	jmp	1f
1:	movw	$0x03d5, %dx
	inb	%dx, %al
	jmp	1f
1:	movb	%al, %ch
	movb	$0x0f, %al
	movw	$0x03d4, %dx
	outb	%al, %dx
	jmp	1f
1:	movw	$0x03d5, %dx
	inb	%dx, %al
	jmp	1f
1:	addb	%al, %cl
	popw	%ax
	ret

/* Ecrit la position du curseur.
 *   ecx: position du curseur
 * modifie: dx */
.align 4
set_cursor_pos:
	pushw	%ax
	movb	$0x0e, %al
	movw	$0x03d4, %dx
	outb	%al, %dx
	jmp	1f
1:	movb	%ch, %al
	movw	$0x03d5, %dx
	outb	%al, %dx
	jmp	1f
1:	movb	$0x0f, %al
	movw	$0x03d4, %dx
	outb	%al, %dx
	jmp	1f
1:	movb	%cl, %al
	movw	$0x03d5, %dx
	outb	%al, %dx
	jmp	1f
1:	popw	%ax
	ret

.globl die
.type die,@function
.align 4
die:
	call	get_cursor_pos
	call	new_line
	leal	error_msg, %ebx
	call	write_str
	movl	4(%esp), %ebx
	call	write_str
	call	new_line
loop:
	hlt
	jmp	loop

	.data

.align 4
msg:
	.asciz	"Decompressing the kernel..."

.align 4
error_msg:
	.asciz	"Failed to decompress the kernel: "

.align 4
stack_top:
	.long	stack + STACK_SIZE
	.word	0x10

	.lcomm	stack, STACK_SIZE

	.section .note.GNU-stack,"",%progbits
