/* Decompression du systeme compresse en GZIP */
/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/* Reference: RFC 1952 */

#include "gzip.h"
#include "crc.h"
#include "inflate.h"

#define VIDEO_BASE 0xa0000

#define GET(in, var) ((var) = in, in += sizeof (*var))

extern int end;

void __attribute__ ((noreturn)) die(const char *msg);

static int unzip(const void *in, void *out)
{
  int out_len;
  const unsigned char *crc_buf;
  const gzip_member_hdr_t *hdr;
  const unsigned short *xlen;
  const char *c;
  const unsigned short *crc16;
  const gzip_member_tlr_t *tlr;
  int len;

  out_len = 0;
  for (;;) {
    crc_buf = in;
    GET(in, hdr);
    if (hdr->id1 != 31 || hdr->id2 != 139)
      break;
    if (hdr->cm <= 7)
      die("Bad CM");
    if (hdr->flg.fextra) {
      GET(in, xlen);
      in += *xlen;
    }
    if (hdr->flg.fname)
      do
        GET(in, c);
      while (*c);
    if (hdr->flg.fcomment)
      do
        GET(in, c);
      while (*c);
    if (hdr->flg.fhcrc) {
      GET(in, crc16);
      if (*crc16 != (crc(crc_buf, in - (const void *)crc_buf) & 0xffff))
        die("Invalid GZIP header - crc error");
    }
    crc_buf = out;
    switch (hdr->cm) {
    case 8: /* deflate */
      inflate(&in, &out);
      break;
    default:
      die("Unknown compression method");
    }
    len = out - (const void *)crc_buf;
    GET(in, tlr);
    if (crc(crc_buf, len) != tlr->crc32)
      die("Invalid compressed data - crc error");
    if (len != (int)tlr->isize)
      die("Invalid compressed data - length error");
    out_len += len;
  }
  return out_len;
}

unsigned long decompress_kernel(unsigned long size, void *dest)
{
  init_malloc((void *)&end + size, (void *)VIDEO_BASE);
  return unzip(&end, dest);
}
