/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef UTIL_H
#define UTIL_H

#define WORD_SIZE 4U

#define ALIGN(addr) (((addr) + (WORD_SIZE - 1)) & ~(WORD_SIZE - 1))

#define NULL ((void *)0)

static inline void *memcpy(void *dest, const void *src, unsigned int n)
{
  int d0, d1, d2;

  asm volatile ("cld\n\t"
                "rep\n\t"
                "movsl\n\t"
                "testb $2, %b4\n\t"
                "je 1f\n\t"
                "movsw\n"
                "1:\ttestb $1, %b4\n\t"
                "je 2f\n\t"
                "movsb\n"
                "2:"
                : "=&c" (d0), "=&D" (d1), "=&S" (d2)
                : "0" (n / 4), "q" (n), "1" (dest), "2" (src)
                : "cc", "memory");
  return dest;
}

static inline void *memmove(void *dest, const void *src, unsigned int n)
{
  int d0, d1, d2;

  if (dest < src)
    asm volatile ("cld\n\t"
                  "rep\n\t"
                  "movsb\n\t"
                  : "=&c" (d0), "=&D" (d1), "=&S" (d2)
                  : "0" (n), "1" (dest), "2" (src)
                  : "cc", "memory");
  else
    asm volatile ("std\n\t"
                  "rep\n\t"
                  "movsb\n\t"
                  : "=&c" (d0), "=&D" (d1), "=&S" (d2)
                  : "0" (n), "1" (dest + n - 1), "2" (src + n - 1)
                  : "cc", "memory");
  return dest;
}

static inline void *memset(void *s, char c, unsigned int n)
{
  int d0, d1;

  asm volatile ("cld\n\t"
                "rep\n\t"
                "stosb"
                : "=&c" (d0), "=&D" (d1)
                : "a" (c), "1" (s), "0" (n)
                : "cc", "memory");
  return s;
}

#endif
