/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "floppy.h"

#include "fat12.h"

#include <string.h>
#include <time.h>

static const unsigned char jmpBoot[3] = {0xeb, 0x3c, 0x90};

bootSector_t bootSector;
unsigned short rootDirSectors;
unsigned short firstDataSector;
unsigned short dataSec;
unsigned short countOfClusters;

void init_volume(unsigned short resvdSecCnt)
{
  memcpy(bootSector.BS_jmpBoot, jmpBoot, 3);
  memcpy(bootSector.BS_OEMName, "XOS     ", 8);
  bootSector.BPB_BytsPerSec = BYTSPERSEC;
  bootSector.BPB_SecPerClus = 1;
  bootSector.BPB_ResvdSecCnt = resvdSecCnt;
  bootSector.BPB_NumFATs = 2;
  bootSector.BPB_RootEntCnt = 224;
  bootSector.BPB_TotSec16 = SECTORS;
  bootSector.BPB_Media = MEDIA;
  bootSector.BPB_FATSz16 = 9;
  bootSector.BPB_SecPerTrk = SECPERTRK;
  bootSector.BPB_NumHeads = HEADS;
  bootSector.BPB_HiddSec = 0;
  bootSector.BPB_TotSec32 = 0;
  bootSector.fat12.BS_DrvNum = 0x00;
  bootSector.fat12.BS_Reserved1 = 0;
  bootSector.fat12.BS_BootSig = 0x29;
  bootSector.fat12.BS_VolID = time(NULL);
  memcpy(bootSector.fat12.BS_VolLab, "NO NAME    ", 11);
  memcpy(bootSector.fat12.BS_FilSysType, "FAT12   ", 8);
  rootDirSectors = (bootSector.BPB_RootEntCnt * 32 + bootSector.BPB_BytsPerSec - 1) / bootSector.BPB_BytsPerSec;
  firstDataSector = bootSector.BPB_ResvdSecCnt + bootSector.BPB_NumFATs * bootSector.BPB_FATSz16 + rootDirSectors;
  dataSec = bootSector.BPB_TotSec16 - (bootSector.BPB_ResvdSecCnt + bootSector.BPB_NumFATs * bootSector.BPB_FATSz16 + rootDirSectors);
  countOfClusters = dataSec / bootSector.BPB_SecPerClus; /* FAT12 */
}
