/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef _ANSI_STDIO_H
#define _ANSI_STDIO_H

#include <bits/stdio.h>
#include <bits/stddef.h>
#include <bits/stdarg.h>

#define _IOFBF 0
#define _IOLBF 1
#define _IONBF 2

#define BUFSIZ 8192

#define EOF (-1)

typedef struct __FILE FILE;

extern FILE *stdin;
extern FILE *stdout;
extern FILE *stderr;
#define stdin stdin
#define stdout stdout
#define stderr stderr

FILE *fopen(const char *filename, const char *mode);
FILE *freopen(const char *filename, const char *mode, FILE *stream);

int fclose(FILE *stream);

int fseek(FILE *stream, long offset, int whence);
long ftell(FILE *stream);
void rewind(FILE *stream);
int fgetpos(FILE *stream, fpos_t *pos);
int fsetpos(FILE *stream, const fpos_t *pos);

size_t fread(void *ptr, size_t size, size_t nmemb, FILE *stream);
size_t fwrite(const void *ptr, size_t size, size_t nmemb, FILE *stream);

int fgetc(FILE *stream);
char *fgets(char *s, int n, FILE *stream);
int getc(FILE *stream);
#define getc(stream) getc(stream)
int getchar();
char *gets(char *s);
int ungetc(int c, FILE *stream);

int fputc(int c, FILE *stream);
int fputs(const char *s, FILE *stream);
int putc(int c, FILE *stream);
#define putc(c, stream) putc(c, stream)
int putchar(int c);
int puts(const char *s);

void clearerr(FILE *stream);
int feof(FILE *stream);
int ferror(FILE *stream);

int fflush(FILE *stream);

void setbuf(FILE *stream, char *buf);
int setvbuf(FILE *stream, char *buf, int mode, size_t size);

int remove(const char *path);

int rename(const char *old, const char *new);

FILE *tmpfile();

char *tmpnam(char *s);

int printf(const char *format, ...) __attribute__ ((format (printf, 1, 2)));
int fprintf(FILE *stream, const char *format, ...) __attribute__ ((format (printf, 2, 3)));
int sprintf(char *s, const char *format, ...) __attribute__ ((format (printf, 2, 3)));
int vprintf(const char *format, va_list ap) __attribute__ ((format (printf, 1, 0)));
int vfprintf(FILE *stream, const char *format, va_list ap) __attribute__ ((format (printf, 2, 0)));
int vsprintf(char *s, const char *format, va_list ap) __attribute__ ((format (printf, 2, 0)));

int scanf(const char *format, ...) __attribute__ ((format (scanf, 1, 2)));
int fscanf(FILE *stream, const char *format, ...) __attribute__ ((format (scanf, 2, 3)));
int sscanf(const char *s, const char *format, ...) __attribute__ ((format (scanf, 2, 3)));

void perror(const char *s);

#endif
