/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef _ANSI_STDLIB_H
#define _ANSI_STDLIB_H

#include <bits/stddef.h>
#include <bits/stdtypes.h>

#define EXIT_FAILURE 1
#define EXIT_SUCCESS 0

#define RAND_MAX     2147483647

#define MB_CUR_MAX   1

typedef struct {
  int quot;
  int rem;
} div_t;

typedef struct {
  long quot;
  long rem;
} ldiv_t;

double atof(const char *str) __attribute__ ((pure, nonnull (1)));
int atoi(const char *str) __attribute__ ((pure, nonnull (1)));
long atol(const char *str) __attribute__ ((pure, nonnull (1)));
double strtod(const char *str, char **endptr) __attribute__ ((nonnull (1)));
long strtol(const char *str, char **endptr, int base) __attribute__ ((nonnull (1)));
unsigned long strtoul(const char *str, char **endptr, int base) __attribute__ ((nonnull (1)));

void *calloc(size_t nmemb, size_t size) __attribute__ ((malloc));
void free(void *ptr);
void *malloc(size_t size) __attribute__ ((malloc));
void *realloc(void *ptr, size_t size) __attribute__ ((warn_unused_result));

void abort() __attribute__ ((noreturn));
int atexit(void (*func)()) __attribute__ ((nonnull (1)));
void exit(int status) __attribute__ ((noreturn));
char *getenv(const char *name) __attribute__ ((nonnull (1)));
int system(const char *command);

void *bsearch(const void *key, const void *base, size_t nitems, size_t size, int (*compar)(const void *, const void *)) __attribute__ ((nonnull (1, 2, 5)));
void qsort(void *base, size_t nitems, size_t size, int (*compar)(const void *, const void *)) __attribute__ ((nonnull (1, 4)));

int abs(int i) __attribute__ ((const));
long labs(long i) __attribute__ ((const));
div_t div(int numer, int denom) __attribute__ ((const));
ldiv_t ldiv(long numer, long denom) __attribute__ ((const));
int rand();
void srand(unsigned int seed);

#endif
