/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef _BITS_STAT_H
#define _BITS_STAT_H

#include <bits/types.h>

#define S_IFMT  0170000 /* file type */

/* file type */
#define S_IFIFO 0010000 /* FIFO special */
#define S_IFCHR 0020000 /* character special */
#define S_IFDIR 0040000 /* directory */
#define S_IFBLK 0060000 /* block special */
#define S_IFREG 0100000 /* regular */

/* file mode bits */
#define S_IRWXU 0000700 /* read, write, execute/search by owner */
#define S_IRUSR 0000400 /* read permission, owner */
#define S_IWUSR 0000200 /* write permission, owner */
#define S_IXUSR 0000100 /* execute/search permission, owner */
#define S_IRWXG 0000700 /* read, write, execute/search by group */
#define S_IRGRP 0000040 /* read permission, group */
#define S_IWGRP 0000020 /* write permission, group */
#define S_IXGRP 0000010 /* execute/search permission, group */
#define S_IRWXO 0000700 /* read, write, execute/search by others */
#define S_IROTH 0000004 /* read permission, others */
#define S_IWOTH 0000002 /* write permission, others */
#define S_IXOTH 0000001 /* execute/search permission, others */

struct stat {
  dev_t     st_dev;     /* device ID of device containing file */
  ino_t     st_ino;     /* file serial number */
  mode_t    st_mode;    /* mode of file */
  dev_t     st_rdev;    /* device ID (if file is character or block special).
                         */
  off_t     st_size;    /* file size in bytes (if file is a regular file) */
  time_t    st_mtime;   /* time of last data modification */
  blksize_t st_blksize; /* a filesystem-specific preferred I/O block size for
                           this object. */
};

#endif
