/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef _UNISTD_H
#define _UNISTD_H

#include <inttypes.h>

#include <bits/stddef.h>
#include <bits/seek.h>
#include <bits/types.h>

#define STDIN_FILENO  0
#define STDOUT_FILENO 1
#define STDERR_FILENO 2

off_t lseek(int fd, off_t offset, int whence);

int close(int fd);

ssize_t read(int fd, void *buf, size_t count);
ssize_t write(int fd, const void *buf, size_t count);

int pipe(int fd[2]);

unsigned int sleep(unsigned int seconds);

int chdir(const char *path) __attribute__ ((nonnull (1)));
char *getcwd(char *buf, size_t size) __attribute__ ((warn_unused_result));

int dup(int fd) __attribute__ ((warn_unused_result));
int dup2(int fd, int newfd);

extern char **environ;

int execve(const char *path, char *const argv[], char *const envp[]) __attribute__ ((nonnull (1)));
int execv(const char *path, char *const argv[]) __attribute__ ((nonnull (1)));
int execle(const char *path, ...) __attribute__ ((nonnull (1)));
int execl(const char *path, ...) __attribute__ ((nonnull (1)));

void _exit(int status) __attribute__ ((noreturn));

pid_t getpid();
pid_t getpgrp();
int setpgid(pid_t pid, pid_t pgid);

pid_t fork();

int isatty(int fd);

int unlink(const char *path) __attribute__ ((nonnull (1)));
int rmdir(const char *path) __attribute__ ((nonnull (1)));

pid_t tcgetpgrp(int fd);
int tcsetpgrp(int fd, pid_t pgid);

int getpagesize() __attribute__ ((const));

int brk(void *addr) __attribute__ ((warn_unused_result));
void *sbrk(intptr_t incr);

#endif
