/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef _XOS_TERMIOS_H
#define _XOS_TERMIOS_H

#define NCCS 11U

#define VINTR   0 /* INTR character */
#define VQUIT   1 /* QUIT character */
#define VERASE  2 /* ERASE character */
#define VKILL   3 /* KILL character */
#define VEOF    4 /* EOF character */
#define VTIME   5 /* TIME value */
#define VMIN    6 /* MIN value */
#define VSTART  7 /* START character */
#define VSTOP   8 /* STOP character */
#define VSUSP   9 /* SUSP character */
#define VEOL   10 /* EOL character */

/* input modes */
#define ICRNL 0x01 /* map CR to NL on input */
#define IGNCR 0x02 /* ignore CR */
#define INLCR 0x04 /* map NL to CR on input */
#define IUCLC 0x08 /* map upper-case to lower-case on input (LEGACY) */
#define IXANY 0x10 /* enable any character to restart output */
#define IXON  0x20 /* enable start/stop output control */

/* output modes */
#define OPOST  0x01 /* post-process output */
#define OLCUC  0x02 /* map lower-case to upper-case on output (LEGACY) */
#define ONLCR  0x04 /* map NL to CR-NL on output */
#define OCRNL  0x08 /* map CR to NL on output */
#define ONOCR  0x10 /* no CR output at column 0 */
#define ONLRET 0x20 /* NL performs CR function */

/* local modes */
#define ECHO     0x01 /* enable echo */
#define ECHOCTL  0x02
#define ECHOE    0x04 /* echo erase character as error-correcting backspace */
#define ECHOK    0x08 /* echo KILL */
#define ECHONL   0x10 /* echo NL */
#define ECHOKE   0x20
#define ICANON   0x40 /* canonical input (erase and kill processing) */
#define ISIG     0x80 /* enable signals */
#define NOFLSH  0x100 /* disable flush after interrupt or quit */
#define TOSTOP  0x200 /* send SIGTTOU for background output */

#endif
