/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <stdlib.h>

#include <signal.h>
#include <unistd.h>

#define ATEXIT_MAX 32

static void (*atexit_func_rooms[ATEXIT_MAX])();
static int atexit_func_count = 0;

int __attribute__ ((weak, visibility ("default"))) atexit(void (*func)())
{
  if (atexit_func_count == ATEXIT_MAX)
    return -1;
  atexit_func_rooms[atexit_func_count++] = func;
  return 0;
}

void __attribute__ ((weak, visibility ("default"))) exit(int status)
{
  while (atexit_func_count > 0)
    atexit_func_rooms[--atexit_func_count]();
  _exit(status);
}

void __attribute__ ((weak, visibility ("default"))) abort(int sig)
{
  raise(SIGABRT);
  _exit(127);
}
