/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <termios.h>

#include "xos.h"
#include "misc.h"

#include <errno.h>

int __attribute__ ((weak, visibility ("default"))) tcgetattr(int fd, struct termios *termios_p)
{
  unsigned long flags;

  if (__syscall_ioctl(fd, _XOS_TGETIF, &flags) == -1)
    return -1;
  termios_p->c_iflag = flags;
  if (__syscall_ioctl(fd, _XOS_TGETOF, &flags) == -1)
    return -1;
  termios_p->c_oflag = flags;
  if (__syscall_ioctl(fd, _XOS_TGETLF, &flags) == -1)
    return -1;
  termios_p->c_lflag = flags;
  if (__syscall_ioctl(fd, _XOS_TGETCC, termios_p->c_cc) == -1)
    return -1;
  return 0;
}

/* POSIX demande que tcsetattr() retourne zero si au moins un des ioctl()
 * reussit. */
int __attribute__ ((weak, visibility ("default"))) tcsetattr(int fd, int optional_actions, const struct termios *termios_p)
{
  unsigned long flags;
  int ok;

  if (optional_actions != TCSANOW && optional_actions != TCSADRAIN) {
    errno = EINVAL;
    return -1;
  }
  if (__syscall_ioctl(fd, _XOS_TGETIF, &flags) == -1)
    return -1; /* fd n'est pas un terminal */
  ok = 0;
  flags = termios_p->c_iflag;
  ok |= __syscall_ioctl(fd, _XOS_TSETIF, &flags) == 0;
  flags = termios_p->c_oflag;
  ok |= __syscall_ioctl(fd, _XOS_TSETOF, &flags) == 0;
  flags = termios_p->c_lflag;
  ok |= __syscall_ioctl(fd, _XOS_TSETLF, &flags) == 0;
  ok |= __syscall_ioctl(fd, _XOS_TSETCC, const_cast(struct termios *, termios_p->c_cc)) == 0;
  if (ok)
    return 0;
  else {
    errno = EINVAL;
    return -1;
  }
}
