/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "builtins.h"

#include "../error.h"
#include "../vars.h"

#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>

static const char builtin_name[] = "exit";

static int get_exit_status_from_arg(const char *arg, int *status)
{
  unsigned int n;

  if (sscanf(arg, "%u", &n) != 1)
    return 0;
  *status = n & 255;
  return 1;
}

static void print_usage(FILE *stream)
{
  fprintf(stream, "Usage: %s [n]\n", builtin_name);
}

static void builtin_print_usage()
{
  print_usage(stdout);
}

static void builtin_print_description()
{
  puts("Cause the shell to exit.");
}

static int builtin_main(int argc, char *argv[])
{
  int c;
  int status;

  while ((c = getopt(argc, argv, "+")) != -1)
    if (c == '?') {
      invalid_option_error(optopt);
      print_usage(stderr);
      return EXIT_FAILURE;
    }

  if (optind < argc) {
    if (!get_exit_status_from_arg(argv[optind], &status)) {
      generic_error("%s: numeric argument required", argv[optind]);
      status = 255;
    }
  }
  else
    status = last_command_status;
  done = 1;
  return status;
}

const struct builtin_struct exit_builtin = {
  .name = builtin_name,
  .print_usage = builtin_print_usage,
  .print_description = builtin_print_description,
  .func = builtin_main
};
