/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "builtins.h"

#include "util.h"
#include "../jobs.h"
#include "../error.h"

#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>

static const char builtin_name[] = "fg";

static void print_usage(FILE *stream)
{
  fprintf(stream, "Usage: %s [job_id]\n", builtin_name);
}

static void builtin_print_usage()
{
  print_usage(stdout);
}

static void builtin_print_description()
{
  puts("Run jobs in the foreground.");
}

static int builtin_main(int argc, char *argv[])
{
  int c;
  struct job_struct *job;
  int status;

  while ((c = getopt(argc, argv, "+")) != -1)
    if (c == '?') {
      invalid_option_error(optopt);
      print_usage(stderr);
      return EXIT_FAILURE;
    }

  if (optind == argc) {
    if (!(job = get_current_job())) {
      generic_error("no current job");
      return EXIT_FAILURE;
    }
  }
  else
    if (!get_job_from_arg(argv[optind], &job)) {
      generic_error("%s: no such job", argv[optind]);
      return EXIT_FAILURE;
    }
  if (run_job_in_foreground(job, &status) == -1)
    return EXIT_FAILURE;
  return status;
}

const struct builtin_struct fg_builtin = {
  .name = builtin_name,
  .print_usage = builtin_print_usage,
  .print_description = builtin_print_description,
  .func = builtin_main
};
