/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "builtins.h"

#include "../error.h"

#include <getopt.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

static const char builtin_name[] = "help";

static void list_command_names()
{
  const struct builtin_struct **builtin;

  for (builtin = builtin_table; *builtin; builtin++)
    printf("%s%c", (*builtin)->name, *(builtin + 1) ? ' ' : '\n');
}

static void print_usage(FILE *stream)
{
  fprintf(stream, "Usage: %s command\n", builtin_name);
  fprintf(stream, "   or: %s -l\n", builtin_name);
}

static void builtin_print_usage()
{
  print_usage(stdout);
}

static void builtin_print_description()
{
  puts("Display helpful information about builtin command.");
}

static int builtin_main(int argc, char *argv[])
{
  int c;
  const struct builtin_struct **builtin;

  while ((c = getopt(argc, argv, "+l")) != -1)
    switch (c) {
    case 'l':
      list_command_names();
      return EXIT_SUCCESS;
    case '?':
      invalid_option_error(optopt);
      print_usage(stderr);
      return EXIT_FAILURE;
    }

  if (optind == argc) {
    generic_error("not enough arguments");
    print_usage(stderr);
    return EXIT_FAILURE;
  }
  for (builtin = builtin_table; *builtin; builtin++)
    if (!strcmp(argv[optind], (*builtin)->name)) {
      (*builtin)->print_usage();
      (*builtin)->print_description();
      return EXIT_SUCCESS;
    }
  generic_error("%s: no help topic", argv[optind]);
  return EXIT_FAILURE;
}

const struct builtin_struct help_builtin = {
  .name = builtin_name,
  .print_usage = builtin_print_usage,
  .print_description = builtin_print_description,
  .func = builtin_main
};
