/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "builtins.h"

#include "util.h"
#include "../jobs.h"
#include "../error.h"

#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>

static const char builtin_name[] = "jobs";

static void print_usage(FILE *stream)
{
  fprintf(stream, "Usage: %s [-l|-p] [job_id...]\n", builtin_name);
}

static void builtin_print_usage()
{
  print_usage(stdout);
}

static void builtin_print_description()
{
  puts("Display status of jobs.");
}

static int builtin_main(int argc, char *argv[])
{
  int format;
  int c;
  int status;
  struct job_struct *job;

  format = RF_NORMAL;
  while ((c = getopt(argc, argv, "+lp")) != -1)
    switch (c) {
    case 'l':
      format = RF_LONG;
      break;
    case 'p':
      format = RF_ONLY_LEADER;
      break;
    case '?':
      invalid_option_error(optopt);
      print_usage(stderr);
      return EXIT_FAILURE;
    }

  if (optind == argc) {
    display_jobs(format);
    return EXIT_SUCCESS;
  }
  else {
    status = EXIT_SUCCESS;
    do {
      if (!get_job_from_arg(argv[optind], &job)) {
        generic_error("%s: no such job", argv[optind]);
        status = EXIT_FAILURE;
        continue;
      }
      display_job(job, format);
    }
    while (++optind < argc);
    return status;
  }
}

const struct builtin_struct jobs_builtin = {
  .name = builtin_name,
  .print_usage = builtin_print_usage,
  .print_description = builtin_print_description,
  .func = builtin_main
};
