/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "builtins.h"

#include "util.h"
#include "../jobs.h"
#include "../error.h"

#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>

static const char builtin_name[] = "wait";

static void print_usage(FILE *stream)
{
  fprintf(stream, "Usage: %s [pid|job_id...]\n", builtin_name);
}

static void builtin_print_usage()
{
  print_usage(stdout);
}

static void builtin_print_description()
{
  puts("Await process completion.");
}

static int builtin_main(int argc, char *argv[])
{
  int c;
  int err;
  int status;
  struct job_struct *job;
  pid_t pid;

  while ((c = getopt(argc, argv, "+")) != -1)
    if (c == '?') {
      invalid_option_error(optopt);
      print_usage(stderr);
      return EXIT_FAILURE;
    }

  if (optind == argc) {
    wait_jobs();
    display_unreported_jobs();
    return EXIT_SUCCESS;
  }
  else {
    err = 0;
    do {
      if (argv[optind][0] == '%') {
        if (!get_job_from_arg(argv[optind], &job)) {
          generic_error("%s: no such job", argv[optind]);
          status = 127;
          continue;
        }
      }
      else {
        if (!get_pid_from_arg(argv[optind], &pid)) {
          generic_error("%s: not a valid process or job ID", argv[optind]);
          err = 1;
          continue;
        }
        if (!(job = get_job_by_pid(pid))) {
          generic_error("pid %d is not a child of this shell", pid);
          err = 1;
          continue;
        }
      }
      wait_job(job, &status);
      display_unreported_jobs();
    }
    while (++optind < argc);
    return err ? EXIT_FAILURE : status;
  }
}

const struct builtin_struct wait_builtin = {
  .name = builtin_name,
  .print_usage = builtin_print_usage,
  .print_description = builtin_print_description,
  .func = builtin_main
};
