/* Copyright (C) 2008  Emmanuel Varoquaux

   This file is part of XOS.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "safe_malloc.h"

#include "error.h"

#include <string.h>
#include <stdlib.h>

static void malloc_error(const char *function_name)
{
  fatal_error("%s: cannot allocate memory", function_name);
}

void *safe_malloc(size_t size)
{
  void *ptr;

  ptr = malloc(size);
  if (size && !ptr)
    malloc_error("malloc");
  return ptr;
}

void *safe_realloc(void *ptr, size_t size)
{
  ptr = realloc(ptr, size);
  if (size && !ptr)
    malloc_error("realloc");
  return ptr;
}

char *safe_strdup(const char *s)
{
  return strcpy(safe_malloc(strlen(s) + 1), s);
}
